
/*
 * Typical pin layout used:
 * -----------------------------------------------------------------------------------------
 *             MFRC522      Arduino       Arduino   Arduino    Arduino          Arduino
 *             Reader/PCD   Uno/101       Mega      Nano v3    Leonardo/Micro   Pro Micro
 * Signal      Pin          Pin           Pin       Pin        Pin              Pin
 * -----------------------------------------------------------------------------------------
 * RST/Reset   RST          9             5         D9         RESET/ICSP-5     RST
 * SPI SS      SDA(SS)      10            53        D10        10               10
 * SPI MOSI    MOSI         11 / ICSP-4   51        D11        ICSP-4           16
 * SPI MISO    MISO         12 / ICSP-1   50        D12        ICSP-1           14
 * SPI SCK     SCK          13 / ICSP-3   52        D13        ICSP-3           15
 */

#include <SPI.h>
#include <MFRC522.h>

constexpr uint8_t RST_PIN = 9;          // Configurable, see typical pin layout above
constexpr uint8_t SS_PIN = 10;         // Configurable, see typical pin layout above

MFRC522 rfid(SS_PIN, RST_PIN);  // Create MFRC522 instance

byte readCard[4];

long s = 0;

long counter(int i_max){
  long output =0;
  for (int i = 0; i <= i_max; i++){
    output+=2^i;
  }
  return output;
}
int animtimer=0;
const byte anim[][8] = {
{
  B00000000,
  B00000000,
  B00000000,
  B00011000,
  B00011000,
  B00000000,
  B00000000,
  B00000000
},{
  B00000000,
  B00000000,
  B00111100,
  B00100100,
  B00100100,
  B00111100,
  B00000000,
  B00000000
},{
  B00000000,
  B01111110,
  B01000010,
  B01000010,
  B01000010,
  B01000010,
  B01111110,
  B00000000
},{
  B11111111,
  B10000001,
  B10000001,
  B10000001,
  B10000001,
  B10000001,
  B10000001,
  B11111111
}};


void scananimation(){
      animtimer = (animtimer+1)%400;
      for (int row=0; row<8; row++){
        digitalWrite(latchPin, LOW);
        shiftOut(dataPin, clockPin, MSBFIRST, 1 << row);
        shiftOut(dataPin, clockPin, MSBFIRST, ~anim[animtimer/100][(row)]);  
        digitalWrite(latchPin, HIGH);
      }
  
}

void scanning(){
  scananimation();  
  if (animtimer<2){
    if (rfid.PICC_IsNewCardPresent() and rfid.PICC_ReadCardSerial()) {
          long id = 1;
          for (byte i = 0; i < rfid.uid.size; i++){
            id *= rfid.uid.uidByte[i];
          }
          Serial.println(id);
          randomSeed(id);
          mkmaze();
          scan=false;
          rfid.PICC_HaltA(); // Stop reading
          
     }
  }
  
}

